# coding:utf-8

=begin rdoc
= Monpeの帳票の埋め込みとPDFの作成を行うライブラリ

== 機能

以下の機能があります。

* Monpe帳票の読み込み
* Rubyスクリプトでのデータ埋め込み
* レイヤーの描画の制御
* 複数帳票のPDFへの出力(PDFの複数ページ化)

== 制限事項

以下の制限があります。

1. 日レセで利用されている以下のオブジェクトのみ対応する
  * 円弧              Standard - Arc
  * 矩形              Standard - Box
  * 楕円              Standard - Ellipse
  * 罫線              Standard - Line
  * 埋め込み画像      Embed - Image
  * 埋め込みテキスト  Embed - Text
  * 丸囲み文字        ORCA - TextCircle

2. 円弧、罫線の矢印は未対応

== 例

簡単な利用例を以下に示します。

  #coding:utf-8
  require 'monpe'
  
  report1 = Monpe::Report.new('test1.red')
  data = report1.get_embed_data
  data['ITEM[1]-1'] = '埋め込み1'
  data['ITEM[2]-1'] = '埋め込み2'
  data['IMAGE[1]-1'] = 'test1.png'
  report1.set_embed_data(data)
  Monpe.export(report1,'test1.pdf')
  
  report2 = Monpe::Report.new('test2.red')
  report2.set_layer_visible('背景',false)
  Monpe.export([report1,report2],'test2.pdf')

=end

module Monpe
end

require 'monpe/version'
require 'monpe/parser'
require 'monpe/paper'
require 'monpe/enum'
require 'monpe/image'
require 'monpe/font'
require 'monpe/text'
require 'monpe/text_line'
require 'monpe/color'
require 'monpe/point'
require 'monpe/rectangle'
require 'monpe/renderer'
require 'monpe/layer'
require 'monpe/util'
require 'monpe/arrow'
require 'monpe/items'
require 'monpe/report'
