# coding:utf-8

module Monpe

  #矢印に対応するクラス
  class Arrow

    ARROW_NONE                     = 0
    ARROW_LINES                    = 1
    ARROW_HOLLOW_TRIANGLE          = 2
    ARROW_FILLED_TRIANGLE          = 3
    ARROW_HOLLOW_DIAMOND           = 4
    ARROW_FILLED_DIAMOND           = 5
    ARROW_HALF_HEAD                = 6
    ARROW_SLASHED_CROSS            = 7
    ARROW_FILLED_ELLIPSE           = 8
    ARROW_HOLLOW_ELLIPSE           = 9
    ARROW_DOUBLE_HOLLOW_TRIANGLE   = 10
    ARROW_DOUBLE_FILLED_TRIANGLE   = 11
    ARROW_UNFILLED_TRIANGLE        = 12
    ARROW_FILLED_DOT               = 13
    ARROW_DIMENSION_ORIGIN         = 14
    ARROW_BLANKED_DOT              = 15
    ARROW_FILLED_BOX               = 16
    ARROW_BLANKED_BOX              = 17
    ARROW_SLASH_ARROW              = 18
    ARROW_INTEGRAL_SYMBOL          = 19
    ARROW_CROW_FOOT                = 20
    ARROW_CROSS                    = 21
    ARROW_FILLED_CONCAVE           = 22
    ARROW_BLANKED_CONCAVE          = 23
    ARROW_ROUNDED                  = 24
    ARROW_HALF_DIAMOND             = 25
    ARROW_OPEN_ROUNDED             = 26
    ARROW_FILLED_DOT_N_TRIANGLE    = 27
    ARROW_ONE_OR_MANY              = 28
    ARROW_NONE_OR_MANY             = 29
    ARROW_ONE_OR_NONE              = 30
    ARROW_ONE_EXACTLY              = 31
    ARROW_BACKSLASH                = 32
    ARROW_THREE_DOTS               = 33

    DEFAULT_ARROW_SIZE             = 0.5
    MIN_ARROW_DIMENSION            = 0.001

    #タイプ
    attr_reader :type

    #長さ
    attr_reader :length

    #幅
    attr_reader :width

    #:call-seq:
    #  new(type,length,width) -> self
    #
    #Arrowを作成する。
    #
    #* 引数 type: テキスト文字列
    #* 引数 length: フォント(Monpe::Font)
    #* 引数 width: 文字列の高さ
    #* 戻り値: self
    def initialize(type,length,width)
      @type      = type
      @length    = length
      @width     = width
    end

    #:nodoc:
    def is_none?
      @type == ARROW_NONE
    end

    #:nodoc:
    def calculate_arrow_point(to,from,line_width)
      line_width = 0.0001 if line_width == 0.0
      @type = ARROW_NONE if @length < MIN_ARROW_DIMENSION or @width < MIN_ARROW_DIMENSION

      move_arrow = to.dup
      move_line = to.dup

      # calc move_arrow
      case @type
      when ARROW_LINES, ARROW_HOLLOW_TRIANGLE, ARROW_UNFILLED_TRIANGLE,ARROW_FILLED_CONCAVE,
           ARROW_BLANKED_CONCAVE, ARROW_DOUBLE_HOLLOW_TRIANGLE
        # not implement
      when ARROW_HALF_HEAD
        # not implement
      when ARROW_FILLED_TRIANGLE, ARROW_HOLLOW_ELLIPSE, ARROW_ROUNDED, ARROW_DIMENSION_ORIGIN,
           ARROW_BLANKED_DOT, ARROW_BLANKED_BOX
        move_arrow.sub(from)
        move_arrow.normalize
        move_arrow.scale(0.5 * line_width)
      when ARROW_ONE_EXACTLY, ARROW_ONE_OR_NONE, ARROW_ONE_OR_MANY, ARROW_NONE_OR_MANY
        # not implement
      end

      # calc move_line
      case @type
      when ARROW_LINES, ARROW_HALF_HEAD
        # not implement
      when ARROW_HOLLOW_TRIANGLE, ARROW_UNFILLED_TRIANGLE, ARROW_FILLED_TRIANGLE,
           ARROW_FILLED_ELLIPSE, ARROW_HOLLOW_ELLIPSE, ARROW_ROUNDED
        move_line = move_arrow.dup
        move_line.normalize
        move_line.scale(@length);
        move_line.add(move_arrow)
      when ARROW_HALF_DIAMOND, ARROW_OPEN_ROUNDED
        # not implement
      when ARROW_HOLLOW_DIAMOND,ARROW_FILLED_DIAMOND
        # not implement
      when ARROW_DIMENSION_ORIGIN,ARROW_BLANKED_DOT,ARROW_BLANKED_BOX
        # not implement
      when ARROW_FILLED_DOT, ARROW_FILLED_BOX
        # not implement
      when ARROW_FILLED_CONCAVE,ARROW_BLANKED_CONCAVE
        # not implement
      when ARROW_DOUBLE_HOLLOW_TRIANGLE
        # not implement
      when ARROW_DOUBLE_FILLED_TRIANGLE
        # not implement
      when ARROW_FILLED_DOT_N_TRIANGLE
        # not implement
      when ARROW_THREE_DOTS
        # not implement
      when ARROW_SLASH_ARROW,ARROW_INTEGRAL_SYMBOL
        # not implement
      when ARROW_ONE_EXACTLY,ARROW_ONE_OR_NONE,ARROW_ONE_OR_MANY,ARROW_NONE_OR_MANY
        # not implement
      end 
      [move_arrow,move_line]
    end

    #:nodoc:
    def draw(renderer,to,from,line_width,color)
      case @type
      when ARROW_DIMENSION_ORIGIN
        # not implement
      when ARROW_INTEGRAL_SYMBOL
        # not implement
      when ARROW_ONE_OR_MANY
        # not implement
      when ARROW_NONE_OR_MANY
        # not implement
      when ARROW_ONE_EXACTLY
        # not implement
      when ARROW_ONE_OR_NONE
        # not implement
      when ARROW_ROUNDED
        # not implement
      when ARROW_OPEN_ROUNDED
        # not implement
      when ARROW_FILLED_DOT_N_TRIANGLE
        # not implement
      when ARROW_THREE_DOTS
        # not implement
      when ARROW_LINES
        # not implement
      when ARROW_HOLLOW_TRIANGLE
        # not implement
      when ARROW_FILLED_TRIANGLE
        draw_filled_triangle(renderer,to,from,line_width,color)
      when ARROW_UNFILLED_TRIANGLE
        # not implement
      when ARROW_HOLLOW_DIAMOND
        # not implement
      when ARROW_FILLED_DIAMOND
        # not implement
      when ARROW_HALF_DIAMOND
        # not implement
      when ARROW_HALF_HEAD
        # not implement
      when ARROW_SLASHED_CROSS
        # not implement
      when ARROW_FILLED_ELLIPSE
        # not implement
      when ARROW_HOLLOW_ELLIPSE
        # not implement
      when ARROW_FILLED_DOT
        # not implement
      when ARROW_DIMENSION_ORIGIN
        # not implement
      when ARROW_BLANKED_DOT
        # not implement
      when ARROW_DOUBLE_HOLLOW_TRIANGLE
        # not implement
      when ARROW_DOUBLE_FILLED_TRIANGLE
        # not implement
      when ARROW_FILLED_DOT_N_TRIANGLE
        # not implement
      when ARROW_FILLED_BOX
        # not implement
      when ARROW_BLANKED_BOX
        # not implement
      when ARROW_SLASH_ARROW
        # not implement
      when ARROW_INTEGRAL_SYMBOL
        # not implement
      when ARROW_CROW_FOOT
        # not implement
      when ARROW_CROSS
        # not implement
      when ARROW_ONE_OR_MANY
        # not implement
      when ARROW_NONE_OR_MANY
        # not implement
      when ARROW_ONE_OR_NONE
        # not implement
      when ARROW_ONE_EXACTLY
        # not implement
      when ARROW_FILLED_CONCAVE
        # not implement
      when ARROW_BLANKED_CONCAVE
        # not implement
      when ARROW_ROUNDED
        # not implement
      when ARROW_OPEN_ROUNDED
        # not implement
      when ARROW_BACKSLASH
        # not implement
      when ARROW_THREE_DOTS
        # not implement
      end
    end

    private

    def draw_filled_triangle(renderer,to,from,line_width,color)
      fill_triangle(renderer,to,from,@length,@width,line_width,nil,color)
      draw_triangle(renderer,to,from,@length,@width,line_width,color,nil)
    end

    def draw_triangle(renderer,to,from,length,width,line_width,fg_color,bg_color)
      poly = calculate_arrow(to,from,length,width)
      renderer.set_linewidth(line_width)
      renderer.set_linestyle(Monpe::LINESTYLE_SOLID)
      renderer.set_linejoin(Monpe::LINEJOIN_MITER)
      renderer.draw_polygon(poly,fg_color)
    end

    def fill_triangle(renderer,to,from,length,width,line_width,fg_color,bg_color)
      poly = calculate_arrow(to,from,length,width)
      renderer.set_fillstyle(Monpe::FILLSTYLE_SOLID)
      renderer.set_linejoin(Monpe::LINEJOIN_MITER)
      renderer.fill_polygon(poly,bg_color)
    end

    def calculate_arrow(to,from,length,width)
      delta = to.dup
      delta.sub(from)
      len = delta.length
      if len <= 0.0001
         delta.x = 1.0
         delta.y = 0.0
      else
         delta.x /= len * 1.0
         delta.y /= len * 1.0
      end
      orth_delta = Point.new(delta.y,-delta.x)
      delta.scale(length)
      orth_delta.scale(width/2.0)

      poly = [to.dup,to.dup,to.dup]
      poly[0].sub(delta)
      poly[0].sub(orth_delta)
      poly[2].sub(delta)
      poly[2].add(orth_delta)
      poly
    end

  end
end
