# coding:utf-8
require 'pango'

module Monpe

  #フォントのクラス
  #
  class Font

    GLOBAL_ZOOM_FACTOR = 20.0

    DIA_FONT_ANY       = 0
    DIA_FONT_SANS      = 1
    DIA_FONT_SERIF     = 2
    DIA_FONT_MONOSPACE = 3

    DIA_FONT_NORMAL  = (0<<2)
    DIA_FONT_OBLIQUE = (1<<2)
    DIA_FONT_ITALIC  = (2<<2)

    DIA_FONT_ULTRALIGHT    = (1<<4)
    DIA_FONT_LIGHT         = (2<<4)
    DIA_FONT_WEIGHT_NORMAL = (0<<4)
    DIA_FONT_MEDIUM        = (3<<4)
    DIA_FONT_DEMIBOLD      = (4<<4)
    DIA_FONT_BOLD          = (5<<4)
    DIA_FONT_ULTRABOLD     = (6<<4)
    DIA_FONT_HEAVY         = (7<<4)

    #フォントファミリー
    attr_reader :family

    #:call-seq:
    #  new(info,height) -> self
    #
    #Fontを作成する。
    #
    #* 引数 info: Monpe::Parserのフォント部分のHash
    #* 引数 height: 高さ
    #* 戻り値: self
    def initialize(family,style,height=1.0)
      @family = family
      @style  = style
      unless @family
        raise NotImplementedError.new
      end
      _set_height(height)
    end

    #:nodoc:
    def get_pango_font_description
      @pfd
    end

    #:nodoc:
    def calc_absolute_size(h)
      size = get_size * (h / get_height)
      @pfd.set_absolute_size(size * Pango::SCALE)
    end

    #:nodoc:
    def set_height(height)
      _set_height(height)
    end

    private

    def _set_height(height)
      @height = height
      @pfd = Pango::FontDescription.new()
      dia_pfd_set_family(dia_font_style_get_family(@style))
      dia_pfd_set_weight(dia_font_style_get_weight(@style))
      dia_pfd_set_slant(dia_font_style_get_slant(@style))
      dia_pfd_set_height(@height)
      @pfd.set_family(@family)
      dia_pfd_set_height(@height)
      @org_size = @pfd.size
    end

    def get_size
      pdu_to_dcm(@org_size)
    end

    def get_height
      @height
    end

    def dia_font_style_get_family(st)
      ((st) & 0x3)
    end

    def dia_font_style_get_slant(st)
      ((st) & (0x3<<2))
    end

    def dia_font_style_get_weight(st)
      ((st) & (0x7<<4))
    end

    def dia_pfd_set_family(fam)
      case fam
      when DIA_FONT_SANS
        @pfd.set_family("sans")
      when DIA_FONT_SERIF
        @pfd.set_family("serif")
      when DIA_FONT_MONOSPACE
        @pfd.set_family("monospace")
      else
        @pfd.set_family("sans")
      end
    end

    def dia_pfd_set_weight(w)
      case w
      when DIA_FONT_ULTRALIGHT
        @pfd.set_weight(Pango::WEIGHT_ULTRALIGHT)
      when DIA_FONT_LIGHT
        @pfd.set_weight(Pango::WEIGHT_LIGHT)
      when DIA_FONT_WEIGHT_NORMAL
        @pfd.set_weight(Pango::WEIGHT_NORMAL)
      when DIA_FONT_MEDIUM
        @pfd.set_weight(500)
      when DIA_FONT_DEMIBOLD
        @pfd.set_weight(600)
      when DIA_FONT_BOLD
        @pfd.set_weight(Pango::WEIGHT_BOLD)
      when DIA_FONT_ULTRABOLD
        @pfd.set_weight(Pango::WEIGHT_ULTRABOLD)
      when DIA_FONT_HEAVY
        @pfd.set_weight(Pango::WEIGHT_HEAVY)
      else
        raise "not reach"
      end
    end

    def dia_pfd_set_slant(sl)
      case sl
      when DIA_FONT_NORMAL
        @pfd.set_style(Pango::STYLE_NORMAL)
      when DIA_FONT_OBLIQUE
        @pfd.set_style(Pango::STYLE_OBLIQUE)
      when DIA_FONT_ITALIC
        @pfd.set_style(Pango::STYLE_ITALIC)
      else
        raise "not reach"
      end
    end

    def dia_pfd_set_height(height)
      @pfd.set_absolute_size(dcm_to_pdu(height) * 0.8)
    end

    def dcm_to_pdu(dcm)
      dcm * Pango::SCALE * GLOBAL_ZOOM_FACTOR
    end

    def pdu_to_dcm(pdu)
      pdu / ( GLOBAL_ZOOM_FACTOR * Pango::SCALE)
    end

  end
end
