# coding:utf-8
require 'gdk_pixbuf2'
require 'cairo'
require 'tempfile'

module Monpe

  #画像のコンテキストを保持するクラス
  #GdkPixbufに対応する画像の読み込みが可能
  class Image
    #GdkPixbuf
    attr_reader :pixbuf
    #画像ファイル名
    attr_reader :filename
    #画像読み込み失敗かどうかのフラグ
    attr_reader :is_broken

    #:call-seq:
    #  new(filename) -> self
    #
    #Imageを作成する。
    #
    #* 引数 filename: 画像のファイルパス
    #* 戻り値: self
    def initialize(filename)
      @filename  = filename
      @is_broken = false
      begin
       @pixbuf = Gdk::Pixbuf.new(filename)
      rescue Exception
       @is_broken = true
       path = Pathname.new(File.expand_path('..',__FILE__))
       broken = path.join('pixmaps').join('broken.png').to_s
       @pixbuf = Gdk::Pixbuf.new(broken)
      end
    end

    #:call-seq:
    #  width -> Integer
    #
    #画像の幅を返す
    #
    #* 戻り値: 画像の幅
    def width
      @pixbuf.width
    end

    #:call-seq:
    #  height -> Integer
    #
    #画像の高さを返す
    #
    #* 戻り値: 画像の高さ
    def height
      @pixbuf.height
    end

    #:call-seq:
    #  create_surface -> Cairo::ImageSurface
    #
    #Gdk::PixbufからCairo::ImageSurfaceを作成する
    #
    #* 戻り値: Cairo::ImageSurface
    def create_surface
      path = Tempfile.new('ruby-monpe-image').path
      pixbuf.save(path,'png')
      Cairo::ImageSurface.from_png(path)
    end

    private

  end
end
