# coding:utf-8

module Monpe
  module Items

    #monpeのArcオブジェクトに対応するクラス
    class Arc < Item
      #--
      # override
      #++

      def initialize(info)
        super(info)
        @endpoints      = @attrs["conn_endpoints"]
        @arc_color      = @attrs["arc_color"]      || Monpe::Color.new(0,0,0)
        @curve_distance = @attrs["curve_distance"] || 0.1
        @line_width     = @attrs["line_width"]     || 0.1
        @line_style     = @attrs["line_style"]     || LINESTYLE_SOLID
        @dashlength     = @attrs["dashlength"]     || DEFAULT_LINESTYLE_DASHLEN
        if @attrs["start_arrow"]
          type   = @attrs["start_arrow"]
          length = @attrs["start_arrow_length"] || DEFAULT_ARROW_LENGTH
          width  = @attrs["start_arrow_width"]  || DEFAULT_ARROW_WIDTH
          @start_arrow = Monpe::Arrow.new(type,length,width)
        else
          @start_arrow = nil
        end
        if @attrs["end_arrow"]
          type   = @attrs["end_arrow"]
          length = @attrs["end_arrow_length"] || DEFAULT_ARROW_LENGTH
          width  = @attrs["end_arrow_width"]  || DEFAULT_ARROW_WIDTH
          @end_arrow = Monpe::Arrow.new(type,length,width)
        else
          @end_arrow = nil
        end

        # update data
        x1 = @endpoints[0].x
        y1 = @endpoints[0].y
        x2 = @endpoints[1].x
        y2 = @endpoints[1].y
        lensq = (x2 - x1) * (x2 - x1) + (y2 - y1) * (y2 - y1)
        @radius = lensq / ( 8 * @curve_distance ) + @curve_distance / 2.0
  
        if lensq == 0.0
          alpha = 1.0
        else
          alpha = (@radius - @curve_distance) / Math.sqrt(lensq)
        end
        xc = (x1 + x2) / 2.0 + (y2 - y1) * alpha
        yc = (y1 + y2) / 2.0 + (x1 - x2) * alpha
        @center = Monpe::Point.new(xc,yc)

        @angle1 = -1.0 * Math.atan2(y1 - yc,x1 - xc) * 180.0 / Math::PI
        @angle1 += 360.0 if @angle1 < 0
        @angle2 = -1.0 * Math.atan2(y2 - yc,x2 - xc) * 180.0 / Math::PI
        @angle2 += 360.0 if @angle2 < 0

        if @radius < 0.0
          @angle2,@angle1 = @angle1,@angle2
        end

        @width = 2 * Monpe.distance_point_point(@center,@endpoints[0])

        mx = (@endpoints[0].x + @endpoints[1].x) / 2.0
        my = (@endpoints[0].y + @endpoints[1].y) / 2.0
        @midpoint = Point.new(mx,my)
        dx = @endpoints[1].x - @endpoints[0].x
        dy = @endpoints[1].y - @endpoints[0].y
        dist = Math.sqrt(dx*dx + dy*dy)
        if dist > 0.000001
          @midpoint.x -= @curve_distance * dy / dist
          @midpoint.y -= @curve_distance * dx / dist
        end
      end

      #:nodoc:
      def render(rdrr)
        #--
        # fixme arrow draw
        #++
        gap0 = @endpoints[0]
        gap1 = @endpoints[1]
        gap2 = @midpoint
        rdrr.set_linewidth(@line_width)
        rdrr.set_linestyle(@line_style)
        rdrr.set_dashlength(@dashlength)
        rdrr.set_linecaps(Monpe::LINECAPS_BUTT)
        if @curve_distance.abs <= 0.01
          rdrr.draw_line(@endpoints[0],@endpoints[1],@arc_color)
        else
          rdrr.draw_arc(@center,@width,@width,@angle1,@angle2,@arc_color)
        end
      end

      private
    end
  end
end
