# coding:utf-8

module Monpe
  module Items
  
    #monpeのBoxオブジェクトに対応するクラス
    #
    class Box < Item

      #:nodoc:
      def initialize(info)
        super(info)
        @elem_corner     = @attrs["elem_corner"]
        @elem_width      = @attrs["elem_width"]
        @elem_height     = @attrs["elem_height"]
        @border_width    = @attrs["border_width"]    || 0.1
        @border_color    = @attrs["border_color"]    || Monpe::Color.new(0,0,0)
        @inner_color     = @attrs["inner_color"]     || Monpe::Color.new(1,1,1)
        @show_background = @attrs["show_background"]
        @line_style      = @attrs["line_style"]      || Monpe::LINESTYLE_SOLID
        @dashlength      = @attrs["dashlength"]      || Monpe::DEFAULT_LINESTYLE_DASHLEN
        @corner_radius   = @attrs["corner_radius"]   || 0.0
        @aspect          = @attrs["aspect"]          || Monpe::FREE_ASPECT
      end
  
      #:nodoc:
      def render(rdrr)
        lr_x = @elem_corner.x + @elem_width
        lr_y = @elem_corner.y + @elem_height
        lr_corner = Monpe::Point.new(lr_x,lr_y)
        rdrr.set_linewidth(@border_width)
        rdrr.set_linestyle(@line_style)
        rdrr.set_dashlength(@dashlength)
        if @corner_radius > 0
          rdrr.set_linejoin(Monpe::LINEJOIN_ROUND)
        else
          rdrr.set_linejoin(Monpe::LINEJOIN_MITER)
        end
        if @show_background
          rdrr.set_fillstyle(Monpe::FILLSTYLE_SOLID)
          if @corner_radius > 0
            rdrr.fill_rounded_rect(@elem_corner,lr_corner,@inner_color,@corner_radius)
          else
            rdrr.fill_rect(@elem_corner,lr_corner,@inner_color)
          end
        end
        if @corner_radius > 0
          rdrr.draw_rounded_rect(@elem_corner,lr_corner,@border_color,@corner_radius)
        else
          rdrr.draw_rect(@elem_corner,lr_corner,@border_color)
        end
      end
  
      private
  
    end
  end
end
