# coding:utf-8

module Monpe
  module Items
  
    #monpeのEllipseオブジェクトに対応するクラス
    #
    class Ellipse < Item
  
      #:nodoc:
      def initialize(info)
        super(info)
        @border_width    = @attrs["border_width"]    || 0.1
        @border_color    = @attrs["border_color"]    || Monpe::Color.new(0,0,0)
        @inner_color     = @attrs["inner_color"]     || Monpe::Color.new(1,1,1)
        @show_background = @attrs["show_background"]
        @show_background = true if @show_background.nil?
        @aspect          = @attrs["aspect"]          || Monpe::FREE_ASPECT
        @line_style      = @attrs["line_style"]      || Monpe::LINESTYLE_SOLID
        @dashlength      = @attrs["dashlength"]      || Monpe::DEFAULT_LINESTYLE_DASHLEN
        @elem_corner     = @attrs["elem_corner"]
        @elem_width      = @attrs["elem_width"]
        @elem_height     = @attrs["elem_height"]
      end

      #:nodoc:
      def render(rdrr)
        cx = @elem_corner.x + @elem_width / 2.0
        cy = @elem_corner.y + @elem_height / 2.0
        center = Monpe::Point.new(cx,cy)
        if @show_background
          rdrr.set_fillstyle(Monpe::FILLSTYLE_SOLID)
          rdrr.fill_ellipse(center,@elem_width,@elem_height,@inner_color)
        end
        rdrr.set_linewidth(@border_width)
        rdrr.set_linestyle(@line_style)
        rdrr.set_dashlength(@dashlength)
        rdrr.draw_ellipse(center,@elem_width,@elem_height,@border_color)
      end
  
      private
  
    end
  end
end
