# coding:utf-8

module Monpe
  module Items

    #monpeのEmbedImageオブジェクトに対応するクラス
    class EmbedImage < EmbedItem

      #:nodoc:
      def initialize(info)
        super(info)
        @elem_corner      = @attrs["elem_corner"]
        @elem_width       = @attrs["elem_width"]
        @elem_height      = @attrs["elem_height"]
        @border_width     = @attrs["border_width"] || 0.1
        @border_color     = @attrs["border_color"] || Monpe::Color.new(0,0,0)
        @line_style       = @attrs["line_style"]   || Monpe::LINESTYLE_SOLID
        @dashlength       = @attrs["dashlength"]   || Monpe::DEFAULT_LINESTYLE_DASHLEN
        @draw_border      = @attrs["draw_border"]
        @keep_aspect      = @attrs["keep_aspect"]
        @keep_orig_aspect = @attrs["keep_orig_aspect"]
        @filename         = @attrs["file"]         || ""
        @image = Monpe::Image.new(@filename)
      end
    
      #:call-seq:
      #  set_data(filename) -> nil
      #
      #埋め込む画像のパスを指定する。
      #
      #* 引数 filename: 埋め込む画像のパス
      #* 戻り値: nil
      def set_data(filename)
        @filename = filename
        @image = Monpe::Image.new(filename)
        nil
      end

      #:call-seq:
      #  get_data -> String
      #
      #埋め込み画像のパスを返す
      #
      #* 戻り値: 埋め込み画像のパス
      def get_data
        @filename
      end
  
      #:nodoc:
      def render(rdrr)
        lrx = @elem_corner.x + @elem_width + @border_width / 2.0
        lry = @elem_corner.y + @elem_height + @border_width / 2.0
        lr = Monpe::Point.new(lrx,lry)
        
        ulx = @elem_corner.x  - @border_width / 2.0
        uly = @elem_corner.y  - @border_width / 2.0
        ul = Monpe::Point.new(ulx,uly)

        if @draw_border
          rdrr.set_linewidth(@border_width)
          rdrr.set_linestyle(@line_style)
          rdrr.set_dashlength(@dashlength)
          rdrr.set_linejoin(Monpe::LINEJOIN_MITER)
          rdrr.draw_rect(ul,lr,@border_color)
        end

        if @keep_orig_aspect
          aspect = @image.width / @image.height * 1.0
          w = @elem_height * aspect
          h = @elem_height
		  if w >= @elem_width
            w = @elem_width
            h = @elem_width * (1.0 / aspect)
          end
          x = @elem_corner.x + @elem_width  / 2.0 - w / 2.0
          y = @elem_corner.y + @elem_height / 2.0 - h / 2.0
          corner = Monpe::Point.new(x,y)
          rdrr.draw_image(corner,w,h,@image)
        else
          rdrr.draw_image(@elem_corner,@elem_width,@elem_height,@image)
        end
      end
  
      private
  
    end
  end
end
