# coding:utf-8

module Monpe
  module Items

    #monpeのEmbedTextオブジェクトに対応するクラス
    #
    class EmbedText < EmbedItem

      #:nodoc:
      def initialize(info)
        super(info)
        @text              = @attrs["text"]
        raise Monpe::ParserError.new("text object needs text attribute") unless @text
        @valign            = @attrs["valign"]            || Monpe::VALIGN_FIRST_LINE
        @fill_color        = @attrs["fill_color"]        || Monpe::Color.new(1,1,1)
        @embed_column_size = @attrs["embed_column_size"] || 0
        @show_background   = @attrs["show_background"]
      end
  
      #:call-seq:
      #  set_data(text) -> nil
      #
      #埋め込みテキストを指定する。
      #
      #* 引数 text: 埋め込みテキスト
      #* 戻り値: nil
      def set_data(text)
        if @embed_column_size > 0
          n = @embed_column_size
          _text = ""
          a = text.split("")
          a.each_with_index do |c,i|
            _text << c
            _text << "\n" if i > 0 and i % n == (n - 1)
          end
          text = _text
        end
        @text.set_text(text)
        nil
      end

      #:call-seq:
      #  get_data -> String
      #
      #埋め込みテキストを指定する。
      #
      #* 戻り値: 埋め込みテキスト
      def get_data
        @text.text
      end
  
      #:nodoc:
      def render(rdrr)
        if @show_background
          bb = rdrr.calc_text_bb(@text)
          ul = Monpe::Point.new(bb.left,bb.top)
          lr = Monpe::Point.new(bb.right,bb.bottom)
          rdrr.fill_rect(ul,lr,@fill_color)
        end
        rdrr.draw_text(@text)
      end
  
      private
  
    end
  end
end
