# coding:utf-8

module Monpe
  module Items

    #不正なフォーマットを示す例外
    class ItemError < StandardError; end

    #monpeのオブジェクト(=Item)に対応するクラス
    #monpeオブジェクトの基底クラス
    class Item

      #オブジェクト位置
      attr_accessor :obj_pos

      #オブジェクトの境界枠
      attr_accessor :obj_bb

      #:call-seq:
      #  new(info) -> self
      #
      #Objectを作成する。
      #このメソッドをユーザが直接使用することは普通ない。
      #
      #* 引数 info: オブジェクトの各種情報のハッシュ
      #* 戻り値: Monpee::Items::Item
      def initialize(info)
        unless info
          raise ItemError.new "invalid info"
        end
        @info = info
        @type    = @info["type"]
        @version = @info["version"]
        @id      = @info["id"]
        @attrs   = @info["attributes"]
        unless @attrs
          raise ItemError.new "invalid info:need attributes"
        end

        @obj_pos = @attrs["obj_pos"]
        unless @obj_pos
          raise ItemError.new "invalid info:need attributes.obj_pos"
        end

        @obj_bb = @attrs["obj_bb"]
        unless @obj_bb
          raise ItemError.new "invalid info:need attributes.obj_bb"
        end
      end

      #:call-seq:
      #  render(renderer) -> nil
      #
      #monpeオブジェクトの描画を行う。
      #このメソッドをユーザ直接使用することは普通ない。
      #
      #* 引数 renderer: Monpe::Rendererオブジェクト
      #* 戻り値: nil
      def render(renderer)
        nil
      end

      #:call-seq:
      #  get_attributes -> Hash
      #
      #monpeオブジェクトの各種情報を返す。
      #
      #* 戻り値: 各種情報のHash
      def get_attributes
        @info
      end

      private

    end
  end
end
