# coding:utf-8

module Monpe
  module Items

    #monpeのLineオブジェクトに対応するクラス
    class Line < Item

      #:nodoc:
      def initialize(info)
        super(info)
        @endpoints     = @attrs["conn_endpoints"]
        @line_color    = @attrs["line_color"]         || Monpe::Color.new(0,0,0)
        @line_width    = @attrs["line_width"]         || 0.1
        @line_style    = @attrs["line_style"]         || Monpe::LINESTYLE_SOLID
        @dashlength    = @attrs["dashlength"]         || Monpe::DEFAULT_LINESTYLE_DASHLEN
        @abs_start_gap = @attrs["absolute_start_gap"] || 0.0
        @abs_end_gap   = @attrs["absolute_end_gap"]   || 0.0

        if @attrs["start_arrow"]
          type   = @attrs["start_arrow"]
          length = @attrs["start_arrow_length"] || DEFAULT_ARROW_LENGTH
          width  = @attrs["start_arrow_width"]  || DEFAULT_ARROW_WIDTH
          @start_arrow = Monpe::Arrow.new(type,length,width)
        else
          @start_arrow = nil
        end
        if @attrs["end_arrow"]
          type   = @attrs["end_arrow"]
          length = @attrs["end_arrow_length"] || DEFAULT_ARROW_LENGTH
          width  = @attrs["end_arrow_width"]  || DEFAULT_ARROW_WIDTH
          @end_arrow = Monpe::Arrow.new(type,length,width)
        else
          @end_arrow = nil
        end
      end
  
      #:nodoc:
      def render(rdrr)
        rdrr.set_linewidth(@line_width)
        rdrr.set_linestyle(@line_style)
        rdrr.set_dashlength(@dashlength)
        rdrr.set_linecaps(Monpe::LINECAPS_BUTT)
        #fixme implement arrow

        if @abs_start_gap != 0.0 or @abs_end_gap != 0.0
          gap = line_adjust_for_absolute_gap
          rdrr.draw_line_with_arrows(gap[0],gap[1],@line_width,@line_color,@start_arrow,@end_arrow)
        else
          rdrr.draw_line_with_arrows(@endpoints[0],@endpoints[1],@line_width,@line_color,@start_arrow,@end_arrow)
        end
      end
  
      private

      def line_adjust_for_absolute_gap
        line_length = Monpe.distance_point_point(@endpoints[0],@endpoints[1])
        g0 = Monpe.point_convex(@endpoints[0],@endpoints[1],1 - @abs_start_gap / line_length)
        g1 = Monpe.point_convex(@endpoints[1],@endpoints[0],1 - @abs_end_gap   / line_length)
        [g0,g1]
      end
  
    end
  end
end
