# coding:utf-8

module Monpe

  #monpeの位置に対応するクラス
  #
  class Point

    #x座標
    attr_accessor :x

    #y座標
    attr_accessor :y

    #:call-seq:
    #  new(x,y) -> self
    #
    #Pointを作成する。
    #
    #* 引数 x: x座標
    #* 引数 y: y座標
    #* 戻り値: self
    def initialize(x,y)
      @x,@y = x,y
    end

    #:nodoc:
    def length
      Math.sqrt(@x*@x*1.0 + @y*@y*1.0)
    end

    #:nodoc:
    def normalize
      len = self.length
      if len > 0.0
        @x /= len * 1.0
        @y /= len * 1.0
      else
        @x = 0.0
        @y = 0.0
      end
    end

    #:nodoc:
    def sub(p)
      @x -= p.x
      @y -= p.y
    end

    #:nodoc:
    def add(p)
      @x += p.x
      @y += p.y
    end

    #:nodoc:
    def scale(alpha)
      @x *= alpha * 1.0
      @y *= alpha * 1.0
    end

    #:nodoc:
    def add_scaled(src,alpha)
      @x += alpha * src.x * 1.0
      @y += alpha * src.y * 1.0
    end

    private
  end

  module_function

  def distance_point_point(p1,p2)
    dx = p1.x - p2.x
    dy = p1.y - p2.y
    Math.sqrt(dx*dx + dy*dy)
  end

  def point_convex(p1,p2,alpha)
    dst = p1.clone
    dst.scale(alpha)
    dst.add_scaled(p2,1.0 - alpha)
    dst
  end

end
