# coding:utf-8

module Monpe

  #テキストに対応するクラス
  class Text
    #テキスト
    attr_reader :text

    #テキストの行の配列
    attr_reader :lines

    #フォント
    attr_reader :font

    #フォントの高さ=文字サイズ
    attr_reader :height

    #位置
    attr_reader :position

    #テキスト前景色
    attr_reader :color

    #水平アライメント
    attr_reader :alignment

    #:call-seq:
    #  new(text,font,height,pos,color,align) -> self
    #
    #Textを作成する。
    #
    #* 引数 text: テキスト文字列
    #* 引数 font: フォント(Monpe::Font)
    #* 引数 height: 文字列の高さ
    #* 引数 pos: テキストの位置
    #* 引数 color: テキストの色
    #* 引数 align: テキストのアライメント
    #* 戻り値: self
    def initialize(text,font,height,pos,color,align)
      @text      = text
      @font      = font
      @height    = height
      @position  = pos
      @color     = color
      @alignment = align
      @lines = []
      @text.split("\n").each do |l|
         @lines << TextLine.new(l,font,height)
      end
    end

    #:call-seq:
    #  get_position -> Point
    #
    #Textの位置を返す
    #
    #* 戻り値: Textの位置
    def get_position
      @position.dup
    end

    #:call-seq:
    #  set_text(text) -> nil
    #
    #テキスト文字列を設定する
    #
    #* 戻り値: nil
    def set_text(text)
      if text
        @text = text.to_s
      else
        @text = ""
      end
      @lines = []
      @text.split("\n").each do |l|
        @lines << TextLine.new(l,font,height)
      end
    end

    private

  end
end
