#coding:utf-8

gem 'minitest'
require 'minitest/autorun'
require 'minitest/unit'
require 'pp'
require 'test_helper'
require 'open3'

class Monpe::TestMonpePrint < MiniTest::Unit::TestCase

  include Monpe::TestHelper

  def setup
  end

  def teardown
  end
  
  def test_monpe_print_noembed
    red = data_file('test_monpe_print.red')
    out = out_file('test_monpe_print_noembed.pdf')
    Open3.popen3("bundle exec ruby -I lib bin/monpe-print #{red} -o #{out}") do |stdin,stdout,stderr,w|
      stdin.close
      puts stdout.read
      st = w.value.exitstatus
      assert_equal(0,st)
    end
    assert(File.exists?(out))
  end

  def test_monpe_print_embed
    red = data_file('test_monpe_print.red')
    out = out_file('test_monpe_print_embed.pdf')
    data = data_file('test_monpe_print.json')
    Open3.popen3("bundle exec ruby -I lib bin/monpe-print #{red} #{data} -o #{out}") do |stdin,stdout,stderr,w|
      stdin.close
      puts stdout.read
      st = w.value.exitstatus
      assert_equal(0,st)
    end
    assert(File.exists?(out))
  end

  def test_monpe_print_show_layers
    red = data_file('test_monpe_print.red')
    out = out_file('test_monpe_print_show_layers.pdf')
    data = data_file('test_monpe_print.json')
    Open3.popen3("bundle exec ruby -I lib bin/monpe-print #{red} #{data} -o #{out} -L back,layer1,layer2") do |stdin,stdout,stderr,w|
      stdin.close
      puts stdout.read
      st = w.value.exitstatus
      assert_equal(0,st)
    end
    assert(File.exists?(out))
  end

  def test_monpe_print_hide_layers
    red = data_file('test_monpe_print.red')
    out = out_file('test_monpe_print_hide_layers.pdf')
    data = data_file('test_monpe_print.json')
    Open3.popen3("bundle exec ruby -I lib bin/monpe-print #{red} #{data} -o #{out} -H layer1,layer2") do |stdin,stdout,stderr,w|
      stdin.close
      puts stdout.read
      st = w.value.exitstatus
      assert_equal(0,st)
    end
    assert(File.exists?(out))
  end

  def test_monpe_print_show_and_hide_layers
    red = data_file('test_monpe_print.red')
    out = out_file('test_monpe_print_show_and_hide_layers.pdf')
    data = data_file('test_monpe_print.json')
    Open3.popen3("bundle exec ruby -I lib bin/monpe-print #{red} #{data} -o #{out} -H layer1,layer2 -L layer1,layer3") do |stdin,stdout,stderr,w|
      stdin.close
      puts stdout.read
      st = w.value.exitstatus
      assert_equal(0,st)
    end
    assert(File.exists?(out))
  end

  def test_monpe_print_offset
    red = data_file('test_monpe_print.red')
    out = out_file('test_monpe_print_offset.pdf')
    data = data_file('test_monpe_print.json')
    Open3.popen3("bundle exec ruby -I lib bin/monpe-print #{red} #{data} -o #{out} -x 1.0 -y 3.0") do |stdin,stdout,stderr,w|
      stdin.close
      puts stdout.read
      st = w.value.exitstatus
      assert_equal(0,st)
    end
    assert(File.exists?(out))
  end

  def test_monpe_print_no_red
    red = data_file('no.red')
    out = out_file('test_monpe_print_no.pdf')
    data = data_file('test_monpe_print.json')
    Open3.popen3("bundle exec ruby -I lib bin/monpe-print #{red} #{data} -o #{out}") do |stdin,stdout,stderr,w|
      stdin.close
      st = w.value.exitstatus
      assert_equal(1,st)
    end
  end

  def test_monpe_print_no_red2
    red = data_file('no.red')
    out = out_file('test_monpe_print_no2.pdf')
    Open3.popen3("bundle exec ruby -I lib bin/monpe-print #{red} -o #{out}") do |stdin,stdout,stderr,w|
      stdin.close
      st = w.value.exitstatus
      assert_equal(1,st)
    end
  end

  def test_monpe_print_short_arg
    Open3.popen3("bundle exec ruby -I lib bin/monpe-print") do |stdin,stdout,stderr,w|
      stdin.close
      st = w.value.exitstatus
      assert_equal(1,st)
    end
  end

  def test_monpe_print_no_data
    red = data_file('test_monpe_print.red')
    out = out_file('test_monpe_print_no_data.pdf')
    data = data_file('no.json')
    Open3.popen3("bundle exec ruby -I lib bin/monpe-print #{red} #{data} -o #{out}") do |stdin,stdout,stderr,w|
      stdin.close
      st = w.value.exitstatus
      assert_equal(1,st)
    end
  end

end
